
# +####################################################################
# ############### Base Model for tensile test with cracks ###############
# #######################################################################
# importing modules
import regionToolset
import job
from sketch import *
import mesh as msh
from mesh import *
from load import *
from interaction import *
from step import *
from assembly import *
from section import *
from material import *
from part import *
from sys import path
import sys
import random
import numpy as np
from numpy import random
from job import *
# Change the path for Site packages
path.append("/home/apadhy/.virtualenvs/abaqus_py/lib/python2.7/site-packages")
import yaml
import os
import math
import copy

################# create parts ###############
def create_part(geometry, model):
    """[summary]

    Arguments:
        geometry {[dict]} -- [subdictionary geometry in Yaml file]
        model {[modelobj]} -- [Model object]
    """
    length = float(geometry["film_length"])   	# Getting length of the composite from dictionary
    height = float(geometry["film_height"])  # Getting height of film from dictionary
    width = float(geometry["film_width"])      # Getting extrusion width of all parts from dictionary

    # Defining the film
    model.ConstrainedSketch(name="film_sketch", sheetSize=200.0) ## Create a sketch named "Mid" with sheet size for display area on screen
    model.sketches["film_sketch"].rectangle(
        point1=(0.0, 0.0), point2=(length, height)) # Create a rectangle with corner points
    model.Part(dimensionality=THREE_D, name="film", type=DEFORMABLE_BODY)
    model.parts["film"].BaseSolidExtrude(
        sketch=model.sketches["film_sketch"], depth=width) # Define part as 3D deformable

	# Defining the assembly
    # Create instances of each part
    model.rootAssembly.DatumCsysByDefault(CARTESIAN)
    model.rootAssembly.Instance(
        dependent=ON, name="film-1", part=model.parts["film"]) #Needs to be a dependent object
# ############### create Materials ###############
def material(materials, model):
    """[Creates materials]
    Arguments:
        materials {[dict]} -- [Dictionary of materials in Yaml]
        model {[Model]} -- [Abaqus model object]
    """
    #Define engineering constants and variables from YAML dictionary
    Density=float(materials["Density"])
    Depvar= int(materials["Depvar"])
    Specific_h= float(materials["Specific_heat"])
    Conductivity= float(materials["Conductivity"])
    User= materials["User_material"]
    print (User)
    model.Material(name="SMA_UM")
    model.materials['SMA_UM'].Density(table=((Density, ), ))
    model.materials['SMA_UM'].Depvar(n=Depvar)
    model.materials['SMA_UM'].Conductivity(table=((Conductivity, ), ))
    model.materials['SMA_UM'].SpecificHeat(table=((Specific_h, ), ))
    # model.materials['SMA_UM'].UserMaterial(mechanicalConstants=(float(User["E"]),float(User["NU"])))

    model.materials['SMA_UM'].UserMaterial(mechanicalConstants=(float(User["IPHASE"]),float(User["MODEL"]),
    float(User["TOL"]), float(User["xi0"]),int(User["NELMTP"]),float(User["EA"]),float(User["EM"]), float(User["nu"]),float(User["alphaA"]),
    float(User["alphaM"]),float(User["Mos"]),float(User["Mof"]),float(User["Aos"]),float(User["Aof"]),float(User["H"]),float(User["rSoMH"]),
    float(User["rSoAH"]),float(User["epstr11"]),float(User["epstr22"]),float(User["epstr33"]),float(User["2epstr23"]),float(User["2epstr13"]),
    float(User["2epstr12"]), float(User["FRULE"])))
## ############### assign sections ###############
def section(geometry, materials, model):
    """[Creates sections for materials and assigns them to element sets chosen at random]

    Arguments:
        geometry {[dict]} -- [Dictionary for geometry from Yaml file]
        materials {[dict]} -- [Dictionary for materials from Yaml file]
        model {[model]} -- [Abaqus model object]
    """
    # define variables used
    width = geometry["film_width"]
    ################################################
    # Create section outer and middle from materials and width
    ####Section upper

    MM = model.parts["film"]
    c = MM.cells
    model.HomogeneousSolidSection(
            material="SMA_UM", name="SMA_Section", thickness=None)
    region = regionToolset.Region(cells=c)
    MM.SectionAssignment(region=region, sectionName="SMA_Section", offset=0.0,
                        offsetType=MIDDLE_SURFACE, offsetField="",
                        thicknessAssignment=FROM_SECTION)
########################  Create SEEDS for MESH Generation  ########################################
############################################
def seeding_old(part,geometry):
    mesh_size = geometry["mesh"]
    Cell_obj= part.cells[0]
    edges=Cell_obj.getEdges() #gets edges indices
    for edge in edges:
        edgelen=round(part.edges[edge].getSize(0),2)
        mesh_div= int(round(edgelen/mesh_size,1))
        edgeobj=(part.edges[edge],)
        part.seedEdgeByNumber(edges=edgeobj, number=mesh_div, constraint=FIXED)
#####################  Generate MESH   ###########################################
#################################################################################
def mesh_old(part,geometry):
    seeding_old(part,geometry)
    elemtype1 = msh.ElemType(elemCode=C3D20R, elemLibrary=STANDARD)
    reg = part.cells
    picked = (reg,)
    part.setElementType(regions=picked, elemTypes=(elemtype1,))
    part.setMeshControls(regions=reg, technique=STRUCTURED)
    part.generateMesh()
################ create step ###############
def step(model,loading):
    """Define load step with solution settings
    Arguments:
        model {[model]} -- Abaqus model object
    """
    analysis = loading["analysis"]
    if analysis==1:
        model.StaticStep(name='Loading', previous='Initial',
            timePeriod=1.0, maxNumInc=1000, initialInc=0.01, minInc=1e-8, maxInc=0.1,nlgeom=OFF)
        model.StaticStep(name='Unload', previous='Loading',
            timePeriod=1.0,maxNumInc=1000, initialInc=0.01, minInc=1e-8, maxInc=0.1,nlgeom=OFF)

    elif analysis==2:
        model.ImplicitDynamicsStep(name='Loading', previous='Initial',
            timePeriod=100.0, maxNumInc=1000, initialInc=0.001, minInc=1e-6,nlgeom=OFF)
        model.ImplicitDynamicsStep(name='Unload', previous='Loading',
            timePeriod=100.0,maxNumInc=1000, initialInc=0.001, minInc=1e-6,nlgeom=OFF)
        # AMPLITUDE###
        ##################
        model.SmoothStepAmplitude(name='Amp-1', timeSpan=STEP, data=((
        0.0, 0.0), (100.0, 1.0)))
        model.SmoothStepAmplitude(name='Amp-2', timeSpan=STEP, data=((
        0.0, 1.0), (100.0, 0.0)))

## ############### create boundary conditions ###############
def BC(loading, geometry, model):
    """Sets boundary conditions for the body
    """
    a = model.rootAssembly
    p= model.parts["film"]

    mesh=geometry["mesh"]/2
    model.rootAssembly.regenerate()
    value = loading["value"]    # Prescribed displacement
    l = geometry["film_length"]
    w = geometry["film_width"]
    h= geometry["film_height"]
    init_temp= loading["init_temp"]
    analysis=loading["analysis"]
    #Left nodes fixed in X
    #####################################
    n0= a.instances["film-1"].nodes.getByBoundingBox(-mesh,-mesh,-mesh,mesh,h+mesh,w+mesh)
    region = regionToolset.Region(nodes=n0)
    a.Set(nodes=n0, name="BC-fix-X")
    model.DisplacementBC(name="BC-fix-X",
                         region=region, u1=0.0, u2=UNSET,createStepName='Initial',
                         u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude='', fixed=OFF,
                         distributionType=UNIFORM, fieldName="", localCsys=None)         # Displacements set

    ####################################################
    # Bottom and top edge nodes fixed in Y and Z
    # n0= a.instances["film-1"].nodes.getByBoundingCylinder((0,0,0),(l-mesh,0,0),mesh)
    # n1= a.instances["film-1"].nodes.getByBoundingCylinder((0,h,w),(l-mesh,h,w),mesh)
    n0= a.instances["film-1"].nodes.getByBoundingSphere((0,0,0),mesh)
    region = regionToolset.Region(nodes=n0)
    a.Set(nodes=n0, name="BC-fix-YZ")
    model.DisplacementBC(name="BC-YZ",
                         region=region, u1=UNSET, u2=0.0,createStepName='Initial',
                         u3=0.0, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude='', fixed=OFF,
                         distributionType=UNIFORM, fieldName="", localCsys=None)

    n1= a.instances["film-1"].nodes.getByBoundingSphere((0,h,0),mesh)
    region = regionToolset.Region(nodes=n1)
    a.Set(nodes=n1, name="BC-fix-Z")
    model.DisplacementBC(name="BC-Z",
                         region=region, u1=UNSET, u2=UNSET,createStepName='Initial',
                         u3=0.0, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude='', fixed=OFF,
                         distributionType=UNIFORM, fieldName="", localCsys=None)

    # Displacements set
    ###############################################
    # Initial temperature assign to all nodes

    n2= a.instances["film-1"].nodes.getByBoundingBox(-mesh,-mesh,-mesh,l+mesh,h+mesh,w+mesh)
    region = regionToolset.Region(nodes=n2)
    a.Set(nodes=n2, name="BC-Temp")
    model.Temperature(name='Predefined Field-1',
        createStepName='Initial', region=region, distributionType=UNIFORM,
        crossSectionDistribution=CONSTANT_THROUGH_THICKNESS, magnitudes=(325.0,
        ))

    # Right faces prescribed displacement/force in X
    #################################################################
    if loading["type"]==1:
        s = a.instances["film-1"].elements.getByBoundingBox(l-mesh*2,-mesh*2,-mesh*2,l+mesh*2,h+mesh*2,w+mesh*2)
        a.Surface(face2Elements=s,name="BC-Surf")
        Surf=a.surfaces["BC-Surf"]
        model.Pressure(name='Load-1', createStepName='Loading',
        region=Surf, distributionType=UNIFORM, field='', magnitude=-value)
        model.loads['Load-1'].setValuesInStep(stepName='Unload', magnitude=0.0)

        if analysis==2:
            model.loads['Load-1'].setValuesInStep(stepName='Loading', amplitude="Amp-1")
            model.loads['Load-1'].setValuesInStep(stepName='Unload', amplitude="Amp-2")

    elif loading["type"]==2:
        s = a.instances["film-1"].nodes.getByBoundingBox(l-mesh,-mesh,-mesh,l+mesh,h+mesh,w+mesh)
        region = regionToolset.Region(nodes=s)
        a.Set(nodes=s, name="BC-Disp")
        model.DisplacementBC(name="BC-Disp", createStepName='Loading',
                            region=region, u1=value, u2=UNSET,
                            u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude=UNSET, fixed=OFF,
                            distributionType=UNIFORM, fieldName="", localCsys=None)
        model.boundaryConditions['BC-Disp'].setValuesInStep(stepName='Unload', u1=0.0)

        if analysis==2:
            model.boundaryConditions['BC-Disp'].setValuesInStep(stepName='Loading', amplitude='Amp-1')
            model.boundaryConditions['BC-Disp'].setValuesInStep(stepName='Unload', amplitude='Amp-2')

        #model.boundaryConditions['Disp-1'].setValuesInStep(stepName='Unload', amplitude='')
# ############### set Output requests ###############
def outputreq(model):
    """Defines output variables to be shown in Visualization after solving
    Arguments:
        model {object} -- model object
    """
    model.FieldOutputRequest(name="F-Output-1" ,createStepName='Loading',
        variables=('S', 'U','RF','SDV'))
    model.FieldOutputRequest(name="F-Output-2",createStepName='Unload',
        variables=('S', 'U','RF','SDV'))
    model.HistoryOutputRequest(name='H-Output-1',
        createStepName='Loading', variables=('IRA1', 'IRA2', 'IRA3', 'IRAR1',
        'IRAR2', 'IRAR3'))
    model.HistoryOutputRequest(name='H-Output-2',
        createStepName='Unload', variables=('IRA1', 'IRA2', 'IRA3', 'IRAR1',
        'IRAR2', 'IRAR3'))
    #'S', 'PE', 'PEEQ', 'PEMAG', 'LE', 'U', 'V', 'A', 'RF','SDV'), numIntervals=50, timeMarks=OFF)
    #####
    # SDV1 Direction of transformation indicator
    # SDV2 Martensitic volume fraction
    # SDV3 Flag for transformation 0 - No 1 - Yes
    # SDV5 Component of transformation strain in 11 direction
    # SDV6 Component of transformation strain in 22 direction
    # SDV7 Component of transformation strain in 33 direction
    # SDV8 2*Component of transformation strain in 23 direction
    # SDV9 2*Component of transformation strain in 13 direction
    # SDV10 2*Component of transformation strain in 12 direction
    # SDV14 Value of effective transformation strain
    # SDV15 Current temperature
    # SDV20 Component of the transformation direction tensor A in 11 direction
    # SDV21 Component of the transformation direction tensor A in 22 direction
    # SDV22 Component of the transformation direction tensor A in 33 direction
    # SDV23 Component of the transformation direction tensor A in 23 direction
    # SDV24 Component of the transformation direction tensor A in 13 direction
    # SDV25 Component of the transformation direction tensor A in 12 direction
    # SDV28 Value of the martensitic volume fraction at point of reversal of transformation
#####################################################
####################################################
# Edit Abaqus input file keywords
def keyword(model):
    model.keywordBlock.synchVersions(storeNodesAndElements=False)
    for n, line in enumerate(model.keywordBlock.sieBlocks):
        if "*Solid Section" in line:
            l=line.split("\n")[0]
            kwds= l+", CONTROLS=HG-1"
            model.keywordBlock.replace(n, kwds)
            kwds1="""*SECTION CONTROLS, NAME=HG-1, HOURGLASS=ENHANCED"""
            model.keywordBlock.insert(n+1, kwds1)
yaml_path = os.path.join(os.getcwd(), "3D_UM_old.yaml")

# load the yaml file and immediatly close it after loading
with open(yaml_path, "r") as stream:
    try:
        input=yaml.safe_load(stream)
    except yaml.YAMLError as exc:
        sys.__stderr__.writelines(exc)
inp_dir= os.path.dirname(yaml_path)
CAE="JOB"              #os.path.join(inp_dir,(os.path.splitext(os.path.basename(yaml_path))[0]))
os.chdir(inp_dir)
model = mdb.models["Model-1"]                # Specify model number
geometry= input["geometry"]
materials= input["materials"]
loading= input["loading"]
### Function calls  #####
create_part(geometry, model)
part= model.parts["film"]
material(materials, model)
section(geometry,materials, model)
mesh_old(part,geometry)
step(model,loading)
outputreq(model)
BC(loading, geometry, model)
keyword(model)
model.rootAssembly.regenerate()
mdb.saveAs(pathName=CAE)
job_name="JOB"               #str(os.path.splitext(os.path.basename(yaml_path))[0])

sub=  os.path.join(os.getcwd(),input["UMAT"])

sys.__stderr__.writelines("JOB IS: {} \n".format(job_name))
mdb.Job(name=job_name, model="Model-1", description="", type=ANALYSIS,
    atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90,
    memoryUnits=PERCENTAGE , getMemoryFromAnalysis=True, #
    explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, echoPrint=OFF,
    modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, userSubroutine=sub,
    scratch="", resultsFormat=ODB, multiprocessingMode=DEFAULT, numCpus=2, numGPUs=0,numDomains=2)
try:
    mdb.jobs[job_name].writeInput(consistencyChecking=OFF)
except Exception as ex:
    sys.__stderr__.writelines("ERROR JOB IS: {} \n".format(job_name))
    sys.__stderr__.writelines(str(ex))
