
# +####################################################################
# ############### Base Model for tensile test with cracks ###############
# #######################################################################
# importing modules
import regionToolset
import job
from sketch import *
import mesh as msh
from mesh import *
from load import *
from interaction import *
from step import *
from assembly import *
from section import *
from material import *
from part import *
from sys import path
import sys
import random
import numpy as np
from numpy import random
from job import *
# Change the path for Site packages
path.append("/home/apadhy/.virtualenvs/abaqus_py/lib/python2.7/site-packages")
import yaml
import os
import math
import copy

################# create parts ###############
def create_part(geometry, model):
    """[summary]

    Arguments:
        geometry {[dict]} -- [subdictionary geometry in Yaml file]
        model {[modelobj]} -- [Model object]
    """
    length = float(geometry["film_length"])   	# Getting length of the composite from dictionary
    height = float(geometry["film_height"])  # Getting height of film from dictionary
    width = float(geometry["film_width"])      # Getting extrusion width of all parts from dictionary

    # Defining the film
    model.ConstrainedSketch(name="film_sketch", sheetSize=200.0) ## Create a sketch named "Mid" with sheet size for display area on screen
    model.sketches["film_sketch"].rectangle(
        point1=(0.0, 0.0), point2=(length, height)) # Create a rectangle with corner points
    model.Part(dimensionality=THREE_D, name="film", type=DEFORMABLE_BODY)
    model.parts["film"].BaseSolidExtrude(
        sketch=model.sketches["film_sketch"], depth=width) # Define part as 3D deformable


	# Defining the assembly
    # Create instances of each part
    model.rootAssembly.DatumCsysByDefault(CARTESIAN)
    model.rootAssembly.Instance(
        dependent=ON, name="film-1", part=model.parts["film"]) #Needs to be a dependent object
# ############### create Materials ###############
def material(materials, model):
    """[Creates materials]
    Arguments:
        materials {[dict]} -- [Dictionary of materials in Yaml]
        model {[Model]} -- [Abaqus model object]
    """
    #Define engineering constants and variables from YAML dictionary
    Density=float(materials["Density"])
    Depvar= int(materials["Depvar"])
    Specific_h= float(materials["Specific_heat"])
    Conductivity= float(materials["Conductivity"])
    User= materials["User_material"]
    print (User)
    model.Material(name="SMA_UM")
    model.materials['SMA_UM'].Density(table=((Density, ), ))
    model.materials['SMA_UM'].Depvar(n=Depvar)
    model.materials['SMA_UM'].Conductivity(table=((Conductivity, ), ))
    model.materials['SMA_UM'].SpecificHeat(table=((Specific_h, ), ))
    #model.materials['SMA_UM'].UserMaterial(mechanicalConstants=(float(User["E"]),float(User["NU"])))

    model.materials['SMA_UM'].UserMaterial(mechanicalConstants=(float(User["IPHASE"]),float(User["MODEL"]),
    float(User["TOL"]), float(User["xi0"]),int(User["NELMTP"]),float(User["EA"]),float(User["EM"]), float(User["nu"]),float(User["alphaA"]),
    float(User["alphaM"]),float(User["Mos"]),float(User["Mof"]),float(User["Aos"]),float(User["Aof"]),float(User["H"]),float(User["rSoMH"]),
    float(User["rSoAH"]),float(User["epstr11"]),float(User["epstr22"]),float(User["epstr33"]),float(User["2epstr23"]),float(User["2epstr13"]),
    float(User["2epstr12"]), float(User["FRULE"])))
## ############### assign sections ###############
def section(geometry,model):
    """[Creates sections for materials and assigns them to element sets chosen at random]

    Arguments:
        geometry {[dict]} -- [Dictionary for geometry from Yaml file]
        materials {[dict]} -- [Dictionary for materials from Yaml file]
        model {[model]} -- [Abaqus model object]
    """
    l=geometry["film_length"]
    h=geometry["film_height"]
    w=geometry["film_width"]
    mesh=geometry["mesh"]
    l_div=int(round(l/mesh,2))
    h_div=int(round(h/mesh,2))
    w_div=int(round(w/mesh,2))

    el_total=l_div*w_div*h_div

    string=""
    for el in range(1,el_total+1):
        string+= "{},".format(el)
        if el%14==0:
            string+="\n"

    model.keywordBlock.synchVersions(storeNodesAndElements=False)
    for n, line in enumerate(model.keywordBlock.sieBlocks):
        if "*End Part" in line:
            kwds1="*Elset, elset=EALL\n"+string+"\n"
            kwds2="** Section: Section-SMA\n"
            kwds3="*Solid Section, elset=EALL, material=SMA_UM,CONTROLS=HG-1\n"
            model.keywordBlock.insert(n-1, kwds1+kwds2+kwds3)
            kwds4="*SECTION CONTROLS, NAME=HG-1, HOURGLASS=ENHANCED"
            model.keywordBlock.insert(n+1, kwds4)


def section_old(geometry, model):
    # define variables used
    ###############################################
    # Create section outer and middle from materials and width
    ###Section upper
    MM = model.parts["film"]
    c = MM.cells
    model.HomogeneousSolidSection(
            material="SMA_UM", name="SMA_Section", thickness=None)
    region = regionToolset.Region(cells=c)
    MM.SectionAssignment(region=region, sectionName="SMA_Section", offset=0.0,
                        offsetType=MIDDLE_SURFACE, offsetField="",
                        thicknessAssignment=FROM_SECTION)
########################  Create SEEDS for MESH Generation  ########################################
def edge_seeding(geometry):
    """[Creates seeds on edges in cells from mesh size]

    """
    l=geometry["film_length"]
    h=geometry["film_height"]
    w=geometry["film_width"]
    mesh=geometry["mesh"]
    l_div=int(round(l/mesh,2))
    h_div=int(round(h/mesh,2))
    w_div=int(round(w/mesh,2))

    n_total= int((l_div+1)*(w_div+1)*(h_div+1))
    el_total=l_div*w_div*h_div

    print ("Total nodes",n_total)
    print ("Total elements",el_total)
    N_coord=[]
    El_coord=dict()
    Node=dict()
    N_index=1
    El=0

    for x in range(0,l_div):
        for z in range(0,w_div):
            for y in range(0,h_div):
                coord=[[mesh*x,mesh*y,mesh*z],[mesh*(x+1),mesh*y,mesh*z],[mesh*(x+1),mesh*(y+1),mesh*z],
                             [mesh*x,mesh*(y+1),mesh*z],[mesh*x,mesh*y,mesh*(z+1)],[mesh*(x+1),mesh*y,mesh*(z+1)],
                             [mesh*(x+1),mesh*(y+1),mesh*(z+1)],[mesh*x,mesh*(y+1),mesh*(z+1)]]
                El+=1
                El_index=[]
                for n in range(len(coord)):
                    if coord[n] not in N_coord:
                        N_coord.append(coord[n])
                        Node[N_index]=coord[n]
                        El_index.append(N_index)
                        N_index+=1
                        #print (coord[n])
                        #print("###")
                    else:
                        for index,coordi in Node.items():
                            if coordi == coord[n]:
                                El_index.append(index)
                El_coord[El]=El_index
    # print(El_coord)
    # print(Node)
    Node_out= ""
    El_out=""
    for key,value in sorted(Node.items()) :
        string= "\t{},\t{}\n".format(key,str(value)[1:-1])
        #print string
        Node_out+=string
    for key,value in sorted(El_coord.items()) :
        string= "\t{},\t{}\n".format(key,str(value)[1:-1])
        #print string
        El_out+=string
    return Node_out,El_out
###########################################
#####################  Generate MESH   ###########################################
def Mesh(part,geometry,model):
    """[Mesh each part]

    Arguments:
        part {[partobj]} -- [Part object]
        elemtype {[str]} -- [Element type]
    """
    Node_out,El_out = edge_seeding(geometry)
    El_type=geometry["El_type"]

    model.keywordBlock.synchVersions(storeNodesAndElements=False)
    for n, line in enumerate(model.keywordBlock.sieBlocks):

        if "*Part" in line:
            l=line.split("\n")[0]
            kwds= l+"\n"+"*Node\n"
            model.keywordBlock.replace(n, kwds)
            kwds1="{}".format(Node_out)+"*Element, type="+El_type+"\n"+"{}\n".format(El_out)
            model.keywordBlock.insert(n, kwds1)



yaml_path = os.path.join(os.getcwd(), "3D_UM.yaml")

# load the yaml file and immediatly close it after loading
with open(yaml_path, "r") as stream:
    try:
        input=yaml.safe_load(stream)
    except yaml.YAMLError as exc:
        sys.__stderr__.writelines(exc)
inp_dir= os.path.dirname(yaml_path)
os.chdir(inp_dir)
model = mdb.models["Model-1"]                # Specify model number
geometry= input["geometry"]
materials= input["materials"]
create_part(geometry, model)
part= model.parts["film"]
material(materials, model)
section(geometry,model)
Mesh(part,geometry,model)
job_name="MESH"
model.rootAssembly.regenerate()
mdb.Job(name=job_name, model="Model-1", description="", type=ANALYSIS,
    atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90,
    memoryUnits=PERCENTAGE , getMemoryFromAnalysis=True, #
    explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, echoPrint=OFF,
    modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, userSubroutine="",
    scratch="", resultsFormat=ODB, multiprocessingMode=DEFAULT, numCpus=2, numGPUs=0,numDomains=2)
try:
    mdb.jobs[job_name].writeInput(consistencyChecking=OFF)
except Exception as ex:
    sys.__stderr__.writelines("ERROR JOB IS: {} \n".format(job_name))
    sys.__stderr__.writelines(str(ex))
