
# +####################################################################
# ############### Base Model for tensile test with cracks ###############
# #######################################################################
# importing modules
import regionToolset
import job
from sketch import *
import mesh as msh
from mesh import *
from load import *
from interaction import *
from step import *
from assembly import *
from section import *
from material import *
from part import *
from sys import path
import sys
import random
import numpy as np
from numpy import random
from job import *
# Change the path for Site packages
path.append("/home/apadhy/.virtualenvs/abaqus_py/lib/python2.7/site-packages")
import yaml
import os
import math
import copy

################ create step ###############
def step(model,loading):
    """Define load step with solution settings
    Arguments:
        model {[model]} -- Abaqus model object
    """
    analysis = loading["analysis"]
    if analysis==1:
        model.StaticStep(name='Loading', previous='Initial',
            timePeriod=1.0, maxNumInc=1000, initialInc=0.001, minInc=1e-6, maxInc=0.1,nlgeom=OFF)
        model.StaticStep(name='Unload', previous='Loading',
            timePeriod=1.0,maxNumInc=10000, initialInc=0.001, minInc=1e-6, maxInc=0.1,nlgeom=OFF)

    elif analysis==2:
        model.ImplicitDynamicsStep(name='Loading', previous='Initial',
            timePeriod=100.0, maxNumInc=1000, initialInc=0.001, minInc=1e-6,nlgeom=OFF)
        model.ImplicitDynamicsStep(name='Unload', previous='Loading',
            timePeriod=100.0,maxNumInc=10000, initialInc=0.001, minInc=1e-6,nlgeom=OFF)
        # AMPLITUDE###
        ##################
        model.SmoothStepAmplitude(name='Amp-1', timeSpan=STEP, data=((
        0.0, 0.0), (100.0, 1.0)))
        model.SmoothStepAmplitude(name='Amp-2', timeSpan=STEP, data=((
        0.0, 1.0), (100.0, 0.0)))
## ############### create boundary conditions ###############
def BC(loading, geometry, model):
    """Sets boundary conditions for the body
    """
    a = model.rootAssembly
    p= model.parts["FILM"]

    mesh=geometry["mesh"]/2
    model.rootAssembly.regenerate()
    value = loading["value"]    # Prescribed displacement
    l = geometry["film_length"]
    w = geometry["film_width"]
    h= geometry["film_height"]
    init_temp= loading["init_temp"]
    analysis = loading["analysis"]

    #Left face nodes fixed in X
    #####################################
    n0= a.instances["FILM-1"].nodes.getByBoundingBox(-mesh,-mesh,-mesh,mesh,h+mesh,w+mesh)
    region = regionToolset.Region(nodes=n0)
    a.Set(nodes=n0, name="BC-fix-X")
    model.DisplacementBC(name="BC-fix-X",
                         region=region, u1=0.0, u2=UNSET,createStepName='Initial',
                         u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude='', fixed=OFF,
                         distributionType=UNIFORM, fieldName="", localCsys=None)         # Displacements set

    ####################################################
    # Bottom and top point nodes fixed in Y and Z
    # n0= a.instances["FILM-1"].nodes.getByBoundingCylinder((0,0,0),(l-mesh,0,0),mesh)
    # n1= a.instances["FILM-1"].nodes.getByBoundingCylinder((0,h,w),(l-mesh,h,w),mesh)
    n0= a.instances["FILM-1"].nodes.getByBoundingSphere((0,0,0),mesh)
    region = regionToolset.Region(nodes=n0)
    a.Set(nodes=n0, name="BC-fix-YZ")
    model.DisplacementBC(name="BC-YZ",
                         region=region, u1=UNSET, u2=0.0,createStepName='Initial',
                         u3=0.0, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude='', fixed=OFF,
                         distributionType=UNIFORM, fieldName="", localCsys=None)

    n1= a.instances["FILM-1"].nodes.getByBoundingSphere((0,h,0),mesh)
    region = regionToolset.Region(nodes=n1)
    a.Set(nodes=n1, name="BC-fix-Z")
    model.DisplacementBC(name="BC-Z",
                         region=region, u1=UNSET, u2=UNSET,createStepName='Initial',
                         u3=0.0, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude='', fixed=OFF,
                         distributionType=UNIFORM, fieldName="", localCsys=None)

    # Displacements set
    ###############################################
    # Initial temperature assign to all nodes

    n2= a.instances["FILM-1"].nodes.getByBoundingBox(-mesh,-mesh,-mesh,l+mesh,h+mesh,w+mesh)
    region = regionToolset.Region(nodes=n2)
    a.Set(nodes=n2, name="BC-Temp")
    model.Temperature(name='Predefined Field-1',
        createStepName='Initial', region=region, distributionType=UNIFORM,
        crossSectionDistribution=CONSTANT_THROUGH_THICKNESS, magnitudes=(init_temp,
        ))
    a.regenerate()

    # Right faces prescribed displacement/force in X
    #################################################################
    if loading["type"]==1:
        s = a.instances["FILM-1"].elements.getByBoundingBox(l-mesh*2,-mesh*2,-mesh*2,l+mesh*2,h+mesh*2,w+mesh*2)
        a.Surface(face4Elements=s,name="BC-Surf")
        Surf=a.surfaces["BC-Surf"]
        model.Pressure(name='Load-1', createStepName='Loading',
        region=Surf, distributionType=UNIFORM, field='', magnitude=-value)
        model.loads['Load-1'].setValuesInStep(stepName='Unload', magnitude=0.0)

        if analysis==2:
            model.loads['Load-1'].setValuesInStep(stepName='Loading', amplitude="Amp-1")
            model.loads['Load-1'].setValuesInStep(stepName='Unload', amplitude="Amp-2")

    elif loading["type"]==2:
        s = a.instances["FILM-1"].nodes.getByBoundingBox(l-mesh,-mesh,-mesh,l+mesh,h+mesh,w+mesh)
        region = regionToolset.Region(nodes=s)
        a.Set(nodes=s, name="BC-Disp")
        model.DisplacementBC(name="BC-Disp", createStepName='Loading',
                            region=region, u1=value, u2=UNSET,
                            u3=UNSET, ur1=UNSET, ur2=UNSET, ur3=UNSET, amplitude=UNSET, fixed=OFF,
                            distributionType=UNIFORM, fieldName="", localCsys=None)
        model.boundaryConditions['BC-Disp'].setValuesInStep(stepName='Unload', u1=0.0)

        if analysis==2:
            model.boundaryConditions['BC-Disp'].setValuesInStep(stepName='Loading', amplitude='Amp-1')
            model.boundaryConditions['BC-Disp'].setValuesInStep(stepName='Unload', amplitude='Amp-2')

# ############### set Output requests ###############
def outputreq(model):
    """Defines output variables to be shown in Visualization after solving
    Arguments:
        model {object} -- model object
    """
    model.FieldOutputRequest(name="F-Output-1" ,createStepName='Loading',
        variables=('S', 'U','RF','SDV'))
    model.FieldOutputRequest(name="F-Output-2",createStepName='Unload',
        variables=('S', 'U','RF','SDV'))
    model.HistoryOutputRequest(name='H-Output-1',
        createStepName='Loading', variables=('IRA1', 'IRA2', 'IRA3', 'IRAR1',
        'IRAR2', 'IRAR3'))
    model.HistoryOutputRequest(name='H-Output-2',
        createStepName='Unload', variables=('IRA1', 'IRA2', 'IRA3', 'IRAR1',
        'IRAR2', 'IRAR3'))


    #'S', 'PE', 'PEEQ', 'PEMAG', 'LE', 'U', 'V', 'A', 'RF','SDV'), numIntervals=50, timeMarks=OFF)
    #####
    # SDV1 Direction of transformation indicator
    # SDV2 Martensitic volume fraction
    # SDV3 Flag for transformation 0 - No 1 - Yes
    # SDV5 Component of transformation strain in 11 direction
    # SDV6 Component of transformation strain in 22 direction
    # SDV7 Component of transformation strain in 33 direction
    # SDV8 2*Component of transformation strain in 23 direction
    # SDV9 2*Component of transformation strain in 13 direction
    # SDV10 2*Component of transformation strain in 12 direction
    # SDV14 Value of effective transformation strain
    # SDV15 Current temperature
    # SDV20 Component of the transformation direction tensor A in 11 direction
    # SDV21 Component of the transformation direction tensor A in 22 direction
    # SDV22 Component of the transformation direction tensor A in 33 direction
    # SDV23 Component of the transformation direction tensor A in 23 direction
    # SDV24 Component of the transformation direction tensor A in 13 direction
    # SDV25 Component of the transformation direction tensor A in 12 direction
    # SDV28 Value of the martensitic volume fraction at point of reversal of transformation
#####################################################
####################################################

yaml_path = os.path.join(os.getcwd(), "3D_UM.yaml")
# load the yaml file and immediatly close it after loading
with open(yaml_path, "r") as stream:
    try:
        input=yaml.safe_load(stream)
    except yaml.YAMLError as exc:
        sys.__stderr__.writelines(exc)

inp_dir= os.path.dirname(yaml_path)
INP=os.path.join(os.getcwd(),"MESH.inp")

CAE="SMA"              #os.path.join(inp_dir,(os.path.splitext(os.path.basename(yaml_path))[0]))
os.chdir(inp_dir)

Mdb()

geometry= input["geometry"]
materials= input["materials"]
loading= input["loading"]
### Function calls  #####
mdb.ModelFromInputFile(name='MESH',
    inputFileName=INP)
model = mdb.models["MESH"]   # Specify model number
step(model,loading)
outputreq(model)
BC(loading, geometry, model)
mdb.saveAs(pathName=CAE)
job_name="SMA"
model.rootAssembly.regenerate()

sub=  os.path.join(os.getcwd(),input["UMAT"])

sys.__stderr__.writelines("JOB IS: {} \n".format(job_name))
mdb.Job(name=job_name, model="MESH", description="", type=ANALYSIS,
    atTime=None, waitMinutes=0, waitHours=0, queue=None, memory=90,
    memoryUnits=PERCENTAGE , getMemoryFromAnalysis=True, #
    explicitPrecision=SINGLE, nodalOutputPrecision=SINGLE, echoPrint=OFF,
    modelPrint=OFF, contactPrint=OFF, historyPrint=OFF, userSubroutine=sub,
    scratch="", resultsFormat=ODB, multiprocessingMode=DEFAULT, numCpus=2, numGPUs=0,numDomains=2)
